@@ -1,4 +1,10 @@
-'''MAVLink X25 CRC code'''
+'''
+MAVLink X25 CRC code
+
+Copyright Andrew Tridgell
+Released under GNU LGPL version 3 or later
+'''
+from builtins import object
 
 
 class x25crc(object):
@@ -24,6 +30,11 @@ class x25crc(object):
         '''add in some more bytes'''
         accum = self.crc
         import array
-        bytes = array.array('B')
-        bytes.fromstring(buf)
-        self.accumulate(bytes)
+        bytes_array = array.array('B')
+        try:  # if buf is bytes
+            bytes_array.frombytes(buf)
+        except TypeError:  # if buf is str
+            bytes_array.frombytes(buf.encode())
+        except AttributeError:  # Python < 3.2
+            bytes_array.fromstring(buf)
+        self.accumulate(bytes_array)
