@@ -10,6 +10,7 @@ from builtins import object
 
 import socket, math, struct, time, os, fnmatch, array, sys, errno
 import select
+import copy
 from pymavlink import mavexpression
 
 # adding these extra imports allows pymavlink to be used directly with pyinstaller
@@ -50,9 +51,9 @@ def mavlink20():
     '''return True if using MAVLink 2.0'''
     return 'MAVLINK20' in os.environ
 
-def evaluate_expression(expression, vars):
+def evaluate_expression(expression, vars, nocondition=False):
     '''evaluation an expression'''
-    return mavexpression.evaluate_expression(expression, vars)
+    return mavexpression.evaluate_expression(expression, vars, nocondition)
 
 def evaluate_condition(condition, vars):
     '''evaluation a conditional (boolean) statement'''
@@ -77,6 +78,25 @@ class location(object):
     def __str__(self):
         return "lat=%.6f,lon=%.6f,alt=%.1f" % (self.lat, self.lng, self.alt)
 
+def add_message(messages, mtype, msg):
+    '''add a msg to array of messages, taking account of instance messages'''
+    if msg._instance_field is None or getattr(msg, msg._instance_field, None) is None:
+        # simple case, no instance field
+        messages[mtype] = msg
+        return
+    instance_value = getattr(msg, msg._instance_field)
+    if not mtype in messages:
+        messages[mtype] = copy.copy(msg)
+        messages[mtype]._instances = {}
+        messages[mtype]._instances[instance_value] = msg
+        messages["%s[%s]" % (mtype, str(instance_value))] = copy.copy(msg)
+        return
+    messages[mtype]._instances[instance_value] = msg
+    prev_instances = messages[mtype]._instances
+    messages[mtype] = copy.copy(msg)
+    messages[mtype]._instances = prev_instances
+    messages["%s[%s]" % (mtype, str(instance_value))] = copy.copy(msg)
+
 def set_dialect(dialect):
     '''set the MAVLink dialect to work with.
     For example, set_dialect("ardupilotmega")
@@ -349,7 +369,7 @@ class mavfile(object):
             # we've seen a new system
             self.sysid_state[src_system] = mavfile_state()
 
-        self.sysid_state[src_system].messages[type] = msg
+        add_message(self.sysid_state[src_system].messages, type, msg)
 
         if src_tuple == radio_tuple:
             # as a special case radio msgs are added for all sysids
@@ -643,9 +663,17 @@ class mavfile(object):
                 return
             mode = mode_map[mode]
         # set mode by integer mode number for ArduPilot
-        self.mav.set_mode_send(self.target_system,
-                               mavlink.MAV_MODE_FLAG_CUSTOM_MODE_ENABLED,
-                               mode)
+        self.mav.command_long_send(self.target_system,
+                                   self.target_component,
+                                   mavlink.MAV_CMD_DO_SET_MODE,
+                                   0,
+                                   mavlink.MAV_MODE_FLAG_CUSTOM_MODE_ENABLED,
+                                   mode,
+                                   0,
+                                   0,
+                                   0,
+                                   0,
+                                   0)
 
     def set_mode_px4(self, mode, custom_mode, custom_sub_mode):
         '''enter arbitrary mode'''
@@ -758,11 +786,6 @@ class mavfile(object):
             self.mav.command_long_send(self.target_system, self.target_component,
                                        mavlink.MAV_CMD_PREFLIGHT_REBOOT_SHUTDOWN, 0,
                                        param1, 0, 0, 0, 0, 0, 0)
-            # send an old style reboot immediately afterwards in case it is an older firmware
-            # that doesn't understand the new convention
-            self.mav.command_long_send(self.target_system, self.target_component,
-                                       mavlink.MAV_CMD_PREFLIGHT_REBOOT_SHUTDOWN, 0,
-                                       1, 0, 0, 0, 0, 0, 0)
 
     def wait_gps_fix(self):
         self.recv_match(type='VFR_HUD', blocking=True)
@@ -1445,6 +1468,8 @@ class mavmmaplog(mavlogfile):
         # mapping from msg id to name
         self.id_to_name = {}
 
+        self.instance_offsets = {}
+
         self.type_nums = None
 
         ofs = 0
@@ -1459,11 +1484,13 @@ class mavmmaplog(mavlogfile):
             if marker == MARKER_V1:
                 mtype = u_ord(self.data_map[ofs+13])
                 mlen += 8
+                data_ofs = 14
             elif marker == MARKER_V2:
                 if ofs+8+10 > self.data_len:
                     break
                 mtype = u_ord(self.data_map[ofs+15]) | (u_ord(self.data_map[ofs+16])<<8) | (u_ord(self.data_map[ofs+17])<<16)
                 mlen += 12
+                data_ofs = 18
                 incompat_flags = u_ord(self.data_map[ofs+10])
                 if incompat_flags & mavlink.MAVLINK_IFLAG_SIGNED:
                     mlen += mavlink.MAVLINK_SIGNATURE_BLOCK_LEN
@@ -1483,7 +1510,18 @@ class mavmmaplog(mavlogfile):
                 self.id_to_name[mtype] = msg.name
                 self.f.seek(ofs)
                 m = self.recv_msg()
-                self.messages[msg.name] = m
+                add_message(self.messages, msg.name, m)
+                if m._instance_field is not None:
+                    self.instance_offsets[mtype] = m._instance_offset
+
+            if mtype in self.instance_offsets:
+                # populate the messages array with a new instance. This assumes we can get the instance
+                # as a single byte integer
+                self.f.seek(ofs + data_ofs + self.instance_offsets[mtype])
+                b = self.f.read(1)
+                instance, = struct.unpack('b', b)
+                mname = self.id_to_name[mtype]
+                self.messages["%s[%s]" % (mname, str(instance))] = self.messages[mname]
 
             self.offsets[mtype].append(ofs)
             self.counts[mtype] += 1
@@ -1880,6 +1918,7 @@ mode_mapping_apm = {
     21 : 'QRTL',
     22 : 'QAUTOTUNE',
     23 : 'QACRO',
+    24 : 'THERMAL',
     }
 mode_mapping_acm = {
     0 : 'STABILIZE',
@@ -2324,7 +2363,7 @@ def dump_message_verbose(f, m):
         timestamp = "%s.%02u: " % (
             time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(timestamp)),
             int(timestamp*100.0)%100)
-    f.write("%s%s (link=%s) (signed=%s) (seq=%u) (src=%u/%u)\n" % (timestamp, m.get_type(), str(m.get_link_id()), str(m.get_signed()), m.get_seq(), m.get_srcSystem(), m.get_srcComponent()))
+    f.write("%s%s (id=%u) (link=%s) (signed=%s) (seq=%u) (src=%u/%u)\n" % (timestamp, m.get_type(), m.get_msgId(), str(m.get_link_id()), str(m.get_signed()), m.get_seq(), m.get_srcSystem(), m.get_srcComponent()))
     for fieldname in m.get_fieldnames():
 
         # format in those most boring way possible:
