@@ -3,8 +3,7 @@
 '''
 search a set of log files for a condition
 '''
-
-import sys, time, os
+from __future__ import print_function
 
 from pymavlink import mavutil
 
@@ -30,7 +29,7 @@ def mavsearch(filename):
         if m is None:
             break
         if mlog.check_condition(args.condition):
-            print m
+            print(m)
             if args.stopcondition:
                 break
         if args.stop:
